%% Author: Aldrin
%% Created: 26 Apr 2012
%% Description: TODO: Add description to main_mon
-module(main_mon).

%%
%% Include files
%%

% REFERENCE: MC ERLANG
-define(debug,true).
-include("macros.hrl").
% /REFERENCE: MC ERLANG

%%
%% Exported Functions
%%
-export([start/3]).

start({M,F,A}, Property, ProcsNames) ->
	SystemPid = spawn(fun() -> receive mon_start -> apply(M,F,A) end end),
	spawn(fun() -> start_monitor(SystemPid, Property, ProcsNames) end).

start_monitor(SystemPid, Property, ProcsNames) ->
	trace([SystemPid], true),
	SystemPid ! mon_start,
	?LOG("Starting monitor ~p",[Property]),
	
	register(?FAIL_MON, self()),
	
	% Allow some time for the system to startup
	timer:sleep(600),
	ResolvedProcs = [whereis(P) || P <- ProcsNames],

	TopMonitor = spawn(fun () -> (Property(ResolvedProcs))([]) end),
    loop_main(TopMonitor).

trace([], _Enable) ->
    ok;
trace([Pid | Pids], Enable) ->
    erlang:trace(Pid, Enable, [set_on_spawn, send, 'receive']),
    trace(Pids, Enable).


%% The main loop of the monitor
%% This function receives all trace messages from the traced processes and communicates with isolated Monitors
%% To reason about the performed actions of the traced process.
loop_main(Monitor) ->
    receive
		fail ->
			io:format("~n~n~n VIOLATION - DETECTED!!! ~n~n~n", []);
%% 			trace(false);
		{trace, Receiver, 'receive', Msg} ->
%% 			?LOG("EAction ~p", [{trace, Receiver, 'receive', Msg}]),
			takeAction(Receiver, Monitor, {recv, Receiver, Msg}),
			loop_main(Monitor);
		{trace, Sender, send, Msg, To} ->    
%% 			?LOG("EAction ~p", [{trace, Sender, send, Msg, To}]),
			takeAction(To, Monitor, {send, Sender, To, Msg}),
			loop_main(Monitor)
    end.



%% Take action functions are needed in order to filter system (Erlang VM) msgs and system events
takeAction(_OtherPid, _Monitor, {recv, _RPid, mon_start}) ->
	ok;
takeAction(_OtherPid, Monitor, Action = {recv, _RPid, Msg}) ->
    case Msg of
        timeout ->
            ok;
        {io_reply, _, _} ->
            ok;
        {X, _} ->
            case m_utils:isSystemProcess(X) of
                true ->
                    ok;
                false ->
					?LOG("Action ~p", [Action]),
                    Monitor ! {action, Action}
            end;
        _ ->
			?LOG("Action ~p", [Action]),
    	    Monitor ! {action, Action}
    end;
takeAction(OtherPid, Monitor, Action = {send, _Pid, Msg}) ->
    case m_utils:isSystemProcess(OtherPid) of
        false ->
            case Msg of
                {io_request, _, _, _} ->
                    ok;
                _ ->
                    	?LOG("Action ~p", [Action]),
						Monitor ! {action, Action}
            end;
        _ -> 
            ok
    end;
takeAction(OtherPid, Monitor, Action) ->
    case m_utils:isSystemProcess(OtherPid) of
        false ->
            	?LOG("Action ~p", [Action]),
				Monitor ! {action, Action};
        _ -> 
            ok
    end.