%% Author: Aldrin
%% Created: 1 Mar 2012
%% Description: TODO: Add description to prop_parser
-module(parsing).

%%
%% Include files
%%
-define(debug,false).
-include("macros.hrl").

-define(LEXER,prop).
-define(PARSER,prop_parser).
-define(MONITOR,detectEr).

%%
%% Exported Functions
%%
-export([leex/0,parse/3]).
-export([lp1/0, lp2/0]).

%%
%% API Functions
%%

leex() ->
	leex:file("..\\src\\prop.xrl"),
	yecc:file("..\\src\\prop_parser.yrl").

parse(FileName, DestFileName, DestModuleName) ->
	{ok, File} = file:read_file(FileName),
    FileContents = binary_to_list(File),
    ?LOG("File Contents: ~p", [FileContents]),
    {ok, Tokens, _EndLine} = ?LEXER:string(FileContents),
    ?LOG("Tokens: ~p", [Tokens]),
	{ok, ParseTree} = ?PARSER:parse(Tokens),
	synthesis_monitor(ParseTree, DestModuleName, DestFileName),
	ParseTree.

synthesis_monitor(ParseTree, ModuleName, DestFileName) ->
	{ok, File} = file:open(DestFileName, [write]),
	{Body, Subjects} = monitor_body(ParseTree, []),
	FormattedSubjects = string:join(Subjects, ", "),
	io:format(File, "~s", ["-module(" ++ ModuleName ++ ").\n\n" ++
	       "-export([property/0]).\n\n" ++
	       "property() -> \n" ++
		   "\t fun([" ++ FormattedSubjects ++ "]) ->\n"]),
    
	io:format(File, "~s", [Body ++ "\nend."]),
	file:close(File),
    compile:file(DestFileName).

monitor_body({false}, Subjects) ->
	Body = "detectEr:m_fls()",
	{Body, Subjects};
monitor_body({nec, {Subj, Op, Value}, Phi}, Subjects) ->
	Body1 =  case Op of
				recv -> "detectEr:m_nec_recv(";
				send -> "detectEr:m_nec_send("
			end,
	Body2 = "\t" ++ Subj ++ "\n"
			"\t" ++ Value ++ "\n",
	{PhiBody, BodyNewSubjects} = monitor_body(Phi, Subjects),
	NewSubjects = case Subj of
						"_" -> BodyNewSubjects;
						_ -> [Subj | BodyNewSubjects]
				  end,
	{Body1 ++ Body2 ++ PhiBody ++ ")", NewSubjects};
monitor_body({and_op, Phi1, Phi2}, Subjects) ->
	BodyStart = "detectEr:m_and(",
	{BodyPhi1, SubjectsPhi1} = monitor_body(Phi1, Subjects),
	{BodyPhi2, SubjectsPhi2} = monitor_body(Phi2, SubjectsPhi1),
	{BodyStart ++ BodyPhi1 ++ BodyPhi2 ++ ")", SubjectsPhi2};
monitor_body({max_op, Var, Phi}, Subjects) ->
	BodyStart = "detectEr:m_max('" ++ Var ++ "', ",
	{BodyPhi, NewSubjects} = monitor_body(Phi, Subjects),
	{BodyStart ++ BodyPhi ++ ")", NewSubjects};
monitor_body({var, Var}, Subjects) ->
	BodyStart = "detectEr:m_var('" ++ Var ++ "')",
	{BodyStart, Subjects}.

lp1() ->
	leex(),
	parse("..\\prop1-addone.txt", "..\\src\\mon_prop_1.erl", "mon_prop_1").
lp2() ->
	leex(),
	parse("..\\prop2-addone.txt", "..\\src\\mon_prop_2.erl", "mon_prop_2").

%%
%% Local Functions
%%

